package cz.zweistein.df.soundsense.gui.control;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BoxLayout;
import javax.swing.DefaultListModel;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.JScrollPane;

import cz.zweistein.autoupdater.AutoUpdater;
import cz.zweistein.autoupdater.callback.IProgressCallback;
import cz.zweistein.df.soundsense.config.ConfigurationXML;

public class SoundPackUpdatePanel extends JPanel implements ActionListener, IProgressCallback {
	private static final long serialVersionUID = 5231590941209603954L;
	
	private ConfigurationXML config;
	private DefaultListModel logModel;
	private JButton startButton;
	private JList log;
	private JLabel downloadLabel;
	private JProgressBar downloadProgressBar;
	private JLabel totalProgressLabel;
	private JProgressBar totalProgressBar;
	private ProgressTicker progressTicker;

	public SoundPackUpdatePanel(ConfigurationXML config) {
		super();
		
		setLayout(new BorderLayout());
		
		this.config = config;
		
		this.progressTicker = new ProgressTicker();
		
		JButton startButton = new JButton("Start automatic update");
		startButton.setIcon(new ImageIcon(Toolkit.getDefaultToolkit().getImage(Icons.TRANSFER)));
		startButton.addActionListener(this);
		this.startButton = startButton;
		
		JPanel headerPanel = new JPanel();
		headerPanel.add(this.progressTicker);
		headerPanel.add(this.startButton);
		
		this.downloadLabel = new JLabel("0 / 0 kb - file ?");
		this.downloadLabel.setAlignmentX(Component.LEFT_ALIGNMENT);
		
		this.downloadProgressBar = new JProgressBar(0, 100);
		this.downloadProgressBar.setStringPainted(true);
		
		this.totalProgressLabel = new JLabel("0 kb / 0 kb ok - total to update is ? kb");
		this.totalProgressLabel.setAlignmentX(Component.LEFT_ALIGNMENT);
		
		this.totalProgressBar = new JProgressBar(0, 100);
		this.totalProgressBar.setStringPainted(true);

		JPanel footerPanel = new JPanel();
		footerPanel.setLayout(new BoxLayout(footerPanel, BoxLayout.PAGE_AXIS));
		
		footerPanel.add(downloadLabel);
		footerPanel.add(downloadProgressBar);
		footerPanel.add(totalProgressLabel);
		footerPanel.add(totalProgressBar);
		
		this.log = new JList();
		this.logModel = new DefaultListModel();
		log.setModel(this.logModel);
		this.add(new JScrollPane(log));
		
		this.add(headerPanel, BorderLayout.PAGE_START);
		this.add(new JScrollPane(log), BorderLayout.CENTER);
		this.add(footerPanel, BorderLayout.PAGE_END);
		
	}

	@Override
	public void actionPerformed(ActionEvent arg0) {
		startButton.setEnabled(false);
		logModel.clear();
		final AutoUpdater updater = new AutoUpdater(config.getSoundpacksPath(), config.getAutoUpdateURL());
		updater.registerCallback(this);
		
		new Thread(new Runnable() {
			
			@Override
			public void run() {
				updater.start();
			}
			
		}).start();

	}
	
	private void logMessage(String message) {
		this.logModel.addElement(message);
		this.log.setSelectedIndex(this.logModel.getSize()-1);
		this.log.ensureIndexIsVisible(this.logModel.getSize()-1);
	}

	@Override
	public void changeFound(String arg0) {
		logMessage("Changed: "+arg0);
	}

	@Override
	public void deleted(String arg0) {
		logMessage("Deleted: "+arg0);
	}

	@Override
	public void done() {
		logMessage("Done, restart SoundSense.");
		startButton.setEnabled(true);
	}

	@Override
	public void error(String arg0) {
		logMessage("Error: "+arg0);
		startButton.setEnabled(true);
	}

	@Override
	public void localParseDone() {
		logMessage("Done scanning "+config.getSoundpacksPath());
	}

	@Override
	public void localParseStart() {
		logMessage("Scanning "+config.getSoundpacksPath());
	}

	@Override
	public void newFound(String arg0) {
		logMessage("New: "+arg0);
	}

	@Override
	public void downloadProgress(Long progress, Long total, String url) {
		
		String text; 
		if (total == null) {
			text = progress/1024+" kb - file "+url;
			
			this.downloadProgressBar.setValue(0);
			this.downloadProgressBar.setString("0%");
			
		} else {
			text = progress/1024+" / "+total/1024+" kb - file "+url;
			
			this.downloadProgressBar.setValue((int)((progress*100)/total));
			this.downloadProgressBar.setString(((progress*100)/total)+"%");
			
		}
		
		this.downloadLabel.setText(text);
	}

	@Override
	public void totalProgress(Long localSize, Long remoteSize) {
		this.totalProgressLabel.setText(localSize/1204+" kb / "+remoteSize/1024 + " kb ok - total to update is "+((remoteSize-localSize)/1204)+" kb");
		this.totalProgressBar.setValue((int)((localSize*100)/remoteSize));
		this.totalProgressBar.setString(((localSize*100)/remoteSize)+"%");
	}

	@Override
	public void tick() {
		this.progressTicker.tick();
	}

}
